@@ -164,7 +164,6 @@ class Actor(WrappedKey):
         return sorted(o)
 
     def __getattr__(self, key):
-
         if self._future and self._future.status not in ("finished", "pending"):
             raise ValueError(
                 "Worker holding Actor was lost.  Status: " + self._future.status
@@ -272,11 +271,11 @@ class BaseActorFuture(abc.ABC, Awaitable[_T]):
 
     @abc.abstractmethod
     def result(self, timeout: str | timedelta | float | None = None) -> _T:
-        ...
+        ...  # pragma: nocover
 
     @abc.abstractmethod
     def done(self) -> bool:
-        ...
+        ...  # pragma: nocover
 
     def __repr__(self) -> Literal["<ActorFuture>"]:
         return "<ActorFuture>"
