@@ -31,6 +31,7 @@ from distributed.dashboard.components.scheduler import (
     TaskStream,
     WorkerNetworkBandwidth,
     WorkersMemory,
+    WorkersTransferBytes,
     WorkerTable,
     events_doc,
     exceptions_doc,
@@ -77,6 +78,7 @@ applications = {
     "/individual-group-progress": individual_doc(TaskGroupProgress, 200),
     "/individual-workers-memory": individual_doc(WorkersMemory, 100),
     "/individual-cluster-memory": individual_doc(ClusterMemory, 100),
+    "/individual-workers-transfer-bytes": individual_doc(WorkersTransferBytes, 100),
     "/individual-cpu": individual_doc(CurrentLoad, 100, fig_attr="cpu_figure"),
     "/individual-nprocessing": individual_doc(
         CurrentLoad, 100, fig_attr="processing_figure"
@@ -117,10 +119,10 @@ applications = {
 
 
 @memoize
-def template_variables():
+def template_variables(scheduler):
     from distributed.diagnostics.nvml import device_get_count
 
-    template_variables: dict = {
+    template_variables = {
         "pages": [
             "status",
             "workers",
@@ -144,6 +146,7 @@ def template_variables():
             if "individual" in x
         ]
         + [{"url": "hardware", "name": "Hardware"}],
+        "jupyter": scheduler.jupyter,
     }
     template_variables["plots"] = sorted(
         template_variables["plots"], key=lambda d: d["name"]
@@ -153,7 +156,10 @@ def template_variables():
 
 def connect(application, http_server, scheduler, prefix=""):
     bokeh_app = BokehApplication(
-        applications, scheduler, prefix=prefix, template_variables=template_variables()
+        applications,
+        scheduler,
+        prefix=prefix,
+        template_variables=template_variables(scheduler),
     )
     application.add_application(bokeh_app)
     bokeh_app.initialize(IOLoop.current())
