@@ -8,7 +8,7 @@ from collections import defaultdict, deque
 from dask.utils import parse_timedelta
 
 from distributed.client import Client
-from distributed.utils import TimeoutError, log_errors
+from distributed.utils import TimeoutError, log_errors, wait_for
 from distributed.worker import get_worker
 
 logger = logging.getLogger(__name__)
@@ -44,7 +44,7 @@ class LockExtension:
                 self.events[name].append(event)
                 future = event.wait()
                 if timeout is not None:
-                    future = asyncio.wait_for(future, timeout)
+                    future = wait_for(future, timeout)
                 try:
                     await future
                 except TimeoutError:
@@ -125,7 +125,7 @@ class Lock:
 
         Returns
         -------
-        True or False whether or not it sucessfully acquired the lock
+        True or False whether or not it successfully acquired the lock
         """
         timeout = parse_timedelta(timeout)
 
