@@ -108,7 +108,7 @@ def info_frame(frame: FrameType) -> dict[str, Any]:
 
 def process(
     frame: FrameType,
-    child,
+    child: object | None,
     state: dict[str, Any],
     *,
     stop: str | None = None,
@@ -325,7 +325,6 @@ def _watch(
     omit: Collection[str],
     stop: Callable[[], bool],
 ) -> None:
-
     recent = create()
     last = time()
 
@@ -521,7 +520,11 @@ def _remove_py_stack(frames):
         yield entry
 
 
-def llprocess(frames, child, state: dict[str, Any] | None) -> dict[str, Any] | None:
+def llprocess(  # type: ignore[no-untyped-def]
+    frames,
+    child: object | None,
+    state: dict[str, Any] | None,
+) -> dict[str, Any] | None:
     """Add counts from low level profile information onto existing state
 
     This uses the ``stacktrace`` module to collect low level stack trace
