@@ -12,31 +12,13 @@ from dask.utils import parse_timedelta
 
 from distributed.compatibility import PeriodicCallback
 from distributed.metrics import time
-from distributed.utils import SyncMethodMixin, log_errors
+from distributed.utils import Deadline, SyncMethodMixin, log_errors, wait_for
 from distributed.utils_comm import retry_operation
 from distributed.worker import get_client, get_worker
 
 logger = logging.getLogger(__name__)
 
 
-class _Watch:
-    def __init__(self, duration=None):
-        self.duration = duration
-        self.started_at = None
-
-    def start(self):
-        self.started_at = time()
-
-    def elapsed(self):
-        return time() - self.started_at
-
-    def leftover(self):
-        if self.duration is None:
-            return None
-        else:
-            return max(0, self.duration - self.elapsed())
-
-
 class SemaphoreExtension:
     """An extension for the scheduler to manage Semaphores
 
@@ -82,15 +64,17 @@ class SemaphoreExtension:
             dask.config.get("distributed.scheduler.locks.lease-validation-interval"),
             default="s",
         )
-        self._pc_lease_timeout = PeriodicCallback(
+        self.scheduler.periodic_callbacks[
+            "semaphore-lease-timeout"
+        ] = pc = PeriodicCallback(
             self._check_lease_timeout, validation_callback_time * 1000
         )
-        self._pc_lease_timeout.start()
+        pc.start()
         self.lease_timeout = parse_timedelta(
             dask.config.get("distributed.scheduler.locks.lease-timeout"), default="s"
         )
 
-    async def get_value(self, name=None):
+    def get_value(self, name=None):
         return len(self.leases[name])
 
     # `comm` here is required by the handler interface
@@ -148,8 +132,7 @@ class SemaphoreExtension:
 
         if isinstance(name, list):
             name = tuple(name)
-        w = _Watch(timeout)
-        w.start()
+        deadline = Deadline.after(timeout)
 
         self.metrics["pending"][name] += 1
         while True:
@@ -157,7 +140,7 @@ class SemaphoreExtension:
                 "Trying to acquire %s for %s with %s seconds left.",
                 lease_id,
                 name,
-                w.leftover(),
+                deadline.remaining,
             )
             # Reset the event and try to get a release. The event will be set if the state
             # is changed and helps to identify when it is worth to retry an acquire
@@ -168,9 +151,7 @@ class SemaphoreExtension:
             # If acquiring fails, we wait for the event to be set, i.e. something has
             # been released and we can try to acquire again (continue loop)
             if not result:
-                future = asyncio.wait_for(
-                    self.events[name].wait(), timeout=w.leftover()
-                )
+                future = wait_for(self.events[name].wait(), timeout=deadline.remaining)
                 try:
                     await future
                     continue
@@ -181,11 +162,11 @@ class SemaphoreExtension:
                 lease_id,
                 name,
                 result,
-                w.elapsed(),
+                deadline.elapsed,
             )
             # We're about to return, so the lease is no longer "pending"
             self.metrics["average_pending_lease_time"][name] = (
-                self.metrics["average_pending_lease_time"][name] + w.elapsed()
+                self.metrics["average_pending_lease_time"][name] + deadline.elapsed
             ) / 2
             self.metrics["pending"][name] -= 1
 
@@ -289,12 +270,6 @@ class Semaphore(SyncMethodMixin):
     acquired. If this happens, a warning is emitted but the internal state is
     not modified.
 
-    .. warning::
-
-        This implementation is still in an experimental state and subtle
-        changes in behavior may occur without any change in the major version
-        of this library.
-
     .. warning::
 
         This implementation is susceptible to lease overbooking in case of
@@ -319,7 +294,7 @@ class Semaphore(SyncMethodMixin):
         When registering, this needs to be awaited.
     scheduler_rpc: ConnectionPool
         The ConnectionPool to connect to the scheduler. If None is provided, it
-        uses the worker or client pool. This paramter is mostly used for
+        uses the worker or client pool. This parameter is mostly used for
         testing.
     loop: IOLoop
         The event loop this instance is using. If None is provided, reuse the
@@ -367,17 +342,20 @@ class Semaphore(SyncMethodMixin):
         scheduler_rpc=None,
         loop=None,
     ):
-
         try:
-            worker = get_worker()
-            self.scheduler = scheduler_rpc or worker.scheduler
-            self.loop = loop or worker.loop
+            try:
+                worker = get_worker()
+                self.scheduler = scheduler_rpc or worker.scheduler
+                self.loop = loop or worker.loop
 
+            except ValueError:
+                client = get_client()
+                self.scheduler = scheduler_rpc or client.scheduler
+                self.loop = loop or client.loop
         except ValueError:
-            client = get_client()
-            self.scheduler = scheduler_rpc or client.scheduler
-            self.loop = loop or client.loop
-
+            # This happens if this is deserialized on the scheduler
+            self.scheduler = None
+            self.loop = None
         self.name = name or "semaphore-" + uuid.uuid4().hex
         self.max_leases = max_leases
         self.id = uuid.uuid4().hex
@@ -405,7 +383,14 @@ class Semaphore(SyncMethodMixin):
 
         # Need to start the callback using IOLoop.add_callback to ensure that the
         # PC uses the correct event loop.
-        self.loop.add_callback(pc.start)
+        if self.loop is not None:
+            self.loop.add_callback(pc.start)
+
+    def _verify_running(self):
+        if not self.scheduler or not self.loop:
+            raise RuntimeError(
+                f"{type(self)} object not properly initialized. This can happen if the object is being deserialized outside of the context of a Client or Worker."
+            )
 
     async def _register(self):
         await retry_operation(
@@ -477,6 +462,7 @@ class Semaphore(SyncMethodMixin):
             Instead of number of seconds, it is also possible to specify
             a timedelta in string format, e.g. "200ms".
         """
+        self._verify_running()
         timeout = parse_timedelta(timeout)
         return self.sync(self._acquire, timeout=timeout)
 
@@ -510,6 +496,7 @@ class Semaphore(SyncMethodMixin):
             immediately, but it will always be automatically released after a specific interval configured using
             "distributed.scheduler.locks.lease-validation-interval" and "distributed.scheduler.locks.lease-timeout".
         """
+        self._verify_running()
         if not self._leases:
             raise RuntimeError("Released too often")
 
@@ -522,9 +509,11 @@ class Semaphore(SyncMethodMixin):
         """
         Return the number of currently registered leases.
         """
+        self._verify_running()
         return self.sync(self.scheduler.semaphore_value, name=self.name)
 
     def __enter__(self):
+        self._verify_running()
         self.acquire()
         return self
 
@@ -532,6 +521,7 @@ class Semaphore(SyncMethodMixin):
         self.release()
 
     async def __aenter__(self):
+        self._verify_running()
         await self.acquire()
         return self
 
@@ -552,6 +542,8 @@ class Semaphore(SyncMethodMixin):
         )
 
     def close(self):
+        self._verify_running()
+        self.refresh_callback.stop()
         return self.sync(self.scheduler.semaphore_close, name=self.name)
 
     def __del__(self):
